;_Toolsammlung.au3

; ----------------------------- Tools -----------------------------------------------------------
; code from outta space....
; #FUNCTION# ====================================================================================================================
; Author ........: Original _ConsoleWrite: jchd@autoitscript.com
; Description ...: Converts characters to windows multi byte layout
; Syntax ........: _convertToMultiByte($sString)
; Parameters ....: $sString           - String to convert
; Return values .: converted string
; Remarks .......: Modified to not output to Console, but to return converted string
; Example .......: No
; ===============================================================================================================================
Func _convertUTF8ToMultiByte($sString)
    Local $aResult = DllCall("kernel32.dll", "int", "WideCharToMultiByte", "uint", 65001, "dword", 0, "wstr", $sString, "int", -1, _
                                "ptr", 0, "int", 0, "ptr", 0, "ptr", 0)
    If @error Then Return SetError(1, @error, 0)
    Local $tText = DllStructCreate("char[" & $aResult[0] & "]")
    $aResult = 		 DllCall("Kernel32.dll", "int", "WideCharToMultiByte", "uint", 65001, "dword", 0, "wstr", $sString, "int", -1, _
								"ptr", DllStructGetPtr($tText), "int", $aResult[0], "ptr", 0, "ptr", 0)
    If @error Then Return SetError(2, @error, 0)
    return DllStructGetData($tText, 1)
EndFunc


; #FUNCTION# ====================================================================================================================
; Name ..........: _GUICtrlListView_CreateArray
; Description ...: Creates a 2-dimensional array from a listview.
; Syntax ........: _GUICtrlListView_CreateArray($hListView[, $sDelimeter = '|'])
; Parameters ....: $hListView           - Control ID/Handle to the control
;                  $sDelimeter          - [optional] One or more characters to use as delimiters (case sensitive). Default is '|'.
; Return values .: Success - The array returned is two-dimensional and is made up of the following:
;                                $aArray[0][0] = Number of rows
;                                $aArray[0][1] = Number of columns
;                                $aArray[0][3] = Delimited string of the column name(s) e.g. Column 1|Column 2|Column 3|Column nth

;                                $aArray[1][0] = 1st row, 1st column
;                                $aArray[1][1] = 1st row, 2nd column
;                                $aArray[1][2] = 1st row, 3rd column
;                                $aArray[n][0] = nth row, 1st column
;                                $aArray[n][1] = nth row, 2nd column
;                                $aArray[n][2] = nth row, 3rd column
; Author ........: guinness
; Remarks .......: GUICtrlListView.au3 should be included.
; Example .......: Yes
; ===============================================================================================================================
Func _GUICtrlListView_CreateArray($hListView, $sDelimeter = '|')
	Local $iColumnCount = _GUICtrlListView_GetColumnCount($hListView), $iDim = 0, $iItemCount = _GUICtrlListView_GetItemCount($hListView)
	If $iColumnCount < 3 Then
		$iDim = 3 - $iColumnCount
	EndIf
	If $sDelimeter = Default Then
		$sDelimeter = '|'
	EndIf

	Local $aColumns = 0, $aReturn[$iItemCount + 1][$iColumnCount + $iDim] = [[$iItemCount, $iColumnCount, '']]
	For $i = 0 To $iColumnCount - 1
		$aColumns = _GUICtrlListView_GetColumn($hListView, $i)
		$aReturn[0][2] &= $aColumns[5] & $sDelimeter
	Next
	$aReturn[0][2] = StringTrimRight($aReturn[0][2], StringLen($sDelimeter))

	For $i = 0 To $iItemCount - 1
		For $j = 0 To $iColumnCount - 1
			$aReturn[$i + 1][$j] = _GUICtrlListView_GetItemText($hListView, $i, $j)
		Next
	Next
	Return SetError(Number($aReturn[0][0] = 0), 0, $aReturn)
EndFunc   ;==>_GUICtrlListView_CreateArray

; #FUNCTION# ====================================================================================================================
; Name...........: _2DArrayInsertRow
; Description ...: Adds a row at the specified position of a 2D array.
; Syntax.........: _2DArrayInsertRow(ByRef $avArray, $iRow)
; Parameters ....: $avArray - Array to modify
;                  $iRow    - Position to insert row at
; Return values .: Success - New size of the first dimension of the array
;                  Failure - 0, sets @error
;                  |1 - $avArray is not an array
;                  |2 - $avArray is not a 2 dimensional array
; Author ........: smartee
; Modified.......:
; Remarks .......:
; Related .......: _ArrayInsert
; Link ..........;
; Example .......;
; ===============================================================================================================================
Func _2DArrayInsertRow(ByRef $avArray, $iRow)
	If Not IsArray($avArray) Then Return SetError(1, 0, 0)
	If UBound($avArray, 0) <> 2 Then Return SetError(2, 0, 0)

	; Add 1 row to the array
	Local $iUBoundRow = UBound($avArray) + 1
	Local $iUBoundCol = UBound($avArray, 2)
	ReDim $avArray[$iUBoundRow][$iUBoundCol]

	; Move all entries down until the specified position
	For $i = $iUBoundRow - 1 To $iRow + 1 Step -1
		For $j = 0 To $iUBoundCol - 1
			$avArray[$i][$j] = $avArray[$i - 1][$j]
		Next
	Next

	Return $iUBoundRow
EndFunc   ;==>_2DArrayInsertRow


;===============================================================================
;		!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!
;		!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!
;		!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!
;		!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!
; Function Name:   _PickIconDialog()
; Description:     Displays a dialog box that allows the user to choose an icon embedded in a resource such as an executable or DLL file
;
; Parameter(s):    $sPath  - Initial path to file
;                  $iIndex - Initial icon to select in the dialog (0-based index)
;                  $hWnd   - [optional] Handle to parent window
;
; Return Value(s): Success - Returns a 2-element array containing the selected path and the selected index
;                  Failure - Returns "" and sets @error
;
; Author(s):       Ultima
;		!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!	!!!!! M O D I F I E D !!!!!
;
Func _FileSaveDialog ($sTitle, $sInitDir, $sFilter = 'All (*.*)', $iOpt = 0, $sDefaultFile = "", $sDefaultExt = "", $mainGUI = 0)
    Local $iFileLen = 65536 ; Max chars in returned string
    ; API flags prepare
    Local $iFlag = BitOR (BitShift (BitAND ($iOpt, 2),-10), BitShift (BitAND ($iOpt,16), 3 ))
    ; Filter string to array convertion
    Local $asFLines = StringSplit ( $sFilter, '|'), $asFilter [$asFLines [0] *2+1]
    Local $i, $iStart, $iFinal, $suFilter = ''
    $asFilter [0] = $asFLines [0] *2
    For $i=1 To $asFLines [0]
        $iStart = StringInStr ($asFLines [$i], '(', 0, 1)
        $iFinal = StringInStr ($asFLines [$i], ')', 0,-1)
        $asFilter [$i*2-1] = StringStripWS (StringLeft ($asFLines [$i], $iStart-1), 3)
        $asFilter [$i*2] = StringStripWS (StringTrimRight (StringTrimLeft ($asFLines [$i], $iStart), StringLen ($asFLines [$i]) -$iFinal+1), 3)
        $suFilter = $suFilter & 'char[' & StringLen ($asFilter [$i*2-1])+1 & '];char[' & StringLen ($asFilter [$i*2])+1 & '];'
    Next
    ; Create API structures
    Local $uOFN = DllStructCreate ('dword;int;int;ptr;ptr;dword;dword;ptr;dword' & _
        ';ptr;int;ptr;ptr;dword;short;short;ptr;ptr;ptr;ptr;ptr;dword;dword' )
    Local $usTitle  = DllStructCreate ('char[' & StringLen ($sTitle) +1 & ']')
    Local $usInitDir= DllStructCreate ('char[' & StringLen ($sInitDir) +1 & ']')
    Local $usFilter = DllStructCreate ($suFilter & 'char')
    Local $usFile   = DllStructCreate ('char[' & $iFileLen & ']')
    Local $usExtn   = DllStructCreate ('char[' & StringLen ($sDefaultExt) +1 & ']')
    For $i=1 To $asFilter [0]
        DllStructSetData ($usFilter, $i, $asFilter [$i])
    Next
    ; Set Data of API structures
    DllStructSetData ($usTitle, 1, $sTitle)
    DllStructSetData ($usInitDir, 1, $sInitDir)
    DllStructSetData ($usFile, 1, $sDefaultFile)
    DllStructSetData ($usExtn, 1, $sDefaultExt)
    DllStructSetData ($uOFN,  1, DllStructGetSize($uOFN))
    DllStructSetData ($uOFN,  2, $mainGUI)
    DllStructSetData ($uOFN,  4, DllStructGetPtr ($usFilter))
    DllStructSetData ($uOFN,  7, 1)
    DllStructSetData ($uOFN,  8, DllStructGetPtr ($usFile))
    DllStructSetData ($uOFN,  9, $iFileLen)
    DllStructSetData ($uOFN, 12, DllStructGetPtr ($usInitDir))
    DllStructSetData ($uOFN, 13, DllStructGetPtr ($usTitle))
    DllStructSetData ($uOFN, 14, $iFlag)
    DllStructSetData ($uOFN, 17, DllStructGetPtr ($usExtn))
    DllStructSetData ($uOFN, 23, BitShift (BitAND ($iOpt, 32), 5))
    ; Call API function
    Local $ret = DllCall ('comdlg32.dll', 'int', 'GetSaveFileName', _
            'ptr', DllStructGetPtr ($uOFN) )
    If $ret [0] Then
        Return DllStructGetData ($usFile, 1)
    Else
        SetError (1)
        Return ""
    EndIf
EndFunc
