#include-once
#include<WindowsConstants.au3>
#include<File.au3>
Global $__goTranslationObject

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_Translate(Const $sString)
; Description ...: Takes a string an looks up the translation in the dictionary.
; Syntax.........: _Translation_Translate
; Parameters ....: $sString - [Const] String to translate
; Return values .: Success - translated string
;                  Failure - original string, sets @error to:
;                  |1 - no transaltion found
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_Translate(Const $sString)
	; Author: ProgAndy
	If Not (IsObj($__goTranslationObject) And $__goTranslationObject.Exists($sString)) Then Return SetError(1,0,$sString)
	Return $__goTranslationObject.Item($sString)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_Translate(Const $sString)
; Description ...: Takes a string an looks up the translation in the dictionary.
; Syntax.........: _Translation_Translate
; Parameters ....: $s      - [Const] String to translate
; Return values .: Success - translated string
;                  Failure - original string, sets @error to:
;                  |1 - no transaltion found
; Author ........: ProgAndy
; Modified.......:
; Remarks .......: the same as _Translation_Translate, just shorter name
; Related .......: _Translation_Translate
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func __(Const $s)
	; Author: ProgAndy
	If Not (IsObj($__goTranslationObject) And $__goTranslationObject.Exists($s)) Then Return SetError(1,0,$s)
	Return $__goTranslationObject.Item($s)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_SI_Load
; Description ...: Loads a language from an ini with multiple languages
; Syntax.........: _Translation_SI_Load(Const $sIniPath, Const $sLanguage)
; Parameters ....: $sIniPath  - the Ini to read from
;                  $sLanguage - the languagen (sectionname)
; Return values .: Success - 1
;                  Failure - 0, sets @error to:
;                  |1 - Could not create dictionary-object
;                  |2 - Could not load language
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_SI_Load(Const $sIniPath, Const $sLanguage)
	; Author: ProgAndy
	If Not IsObj($__goTranslationObject) Then $__goTranslationObject = ObjCreate("Scripting.Dictionary")
	If Not IsObj($__goTranslationObject) Then Return SetError(1,0,0)
	$__goTranslationObject.CompareMode = 1 ; Text
	Local $aSection = IniReadSection($sIniPath, $sLanguage)
	If @error Then Return SetError(2,0,0)
	$__goTranslationObject.RemoveAll( )
	For $i = 1 To $aSection[0][0]
		If not $__goTranslationObject.Exists($aSection[$i][0]) then $__goTranslationObject.Add($aSection[$i][0], $aSection[$i][1])
	Next
	Return SetExtended($aSection[0][0], 1)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_SI_ListAll
; Description ...: Lists all languages in an INI-file
; Syntax.........: _Translation_SI_ListAll(Const $sIniPath)
; Parameters ....: $sIniPath - ini-file
; Return values .: Success - array with all languages (1-based, [0] contains number of languages)
;                  Failure - 0, sets @error to:
;                  |1 - could not read names
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_SI_ListAll(Const $sIniPath)
	; Author: ProgAndy
	Local $aSection = IniReadSectionNames($sIniPath)
	If @error Then Return SetError(1,0,0)
	Return $aSection
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_SI_Choose
; Description ...: Shows a GUI to let the user choose a translation
; Syntax.........: _Translation_SI_Choose(ByRef Const $aTranslations [, $iStart=1])
; Parameters ....: $aTranslations - [ByRef] Array of languages
;                  $iStart        - [optional] Startindex for reading the array (default: 1)
; Return values .: Success - the language
;                  Failure - 0, sets @error to:
;                  |1 - no language chosen or no lanugages given
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_SI_Choose(ByRef Const $aTranslations, $iStart=1)
	; Author: ProgAndy
	If Not (UBound($aTranslations,0) = 1) Or $iStart < 0 Or UBound($aTranslations)<= $iStart Then Return SetError(1,0,'')
	If UBound($aTranslations)-$iStart = 1 Then Return $aTranslations[$iStart]
	Local $iOldOpt = Opt("GUIOnEventMode", 0)
	Local $iOldOptChar = Opt("GUIDataSeparatorChar", Chr(2))
	Local $hGUI = GUICreate("Choose Translation", 200, 100, -1, -1, BitOR($WS_CAPTION, $WS_POPUP, $WS_SYSMENU)), $uMsg, $sTranslation=''
	GUICtrlCreateLabel("Choose translation:", 10, 10, 180, 20)
	Local $iCombo = GUICtrlCreateCombo($aTranslations[$iStart], 10, 30, 180, 20)
	For $i = $iStart+1 To UBound($aTranslations)-1
		GUICtrlSetData($iCombo, $aTranslations[$i])
	Next
	Opt("GUIDataSeparatorChar", $iOldOptChar)
	Local $btnOK = GUICtrlCreateButton("&OK", 10, 60, 85, 25, 0x0001)
	Local $btnCancel = GUICtrlCreateButton("&Cancel", 105, 60, 85, 25)

	GUISetState()
	While 1
		$uMsg = GUIGetMsg(1)
		If $uMsg[1] = $hGUI Then
			Switch $uMsg[0]
				Case -3, $btnCancel
					ExitLoop
				Case $btnOK
					$sTranslation = GUICtrlRead($iCombo)
					ExitLoop
			EndSwitch
		EndIf
	WEnd
	Opt("GUIOnEventMode", $iOldOpt )
	GUIDelete($hGUI)
	Return SetError($sTranslation='', 0, $sTranslation)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_MI_ListAll
; Description ...: Lists all languages stored as multiple ini-files in one directory
; Syntax.........: _Translation_MI_ListAll(Const $sTransPath, Const $sExtension='ini')
; Parameters ....: $sTransPath - Path with the INI-files
;                  $sExtension - [optional] the extension of the files (default: 'ini')
; Return values .: Success - 2D-Array containing the ini-file and the language name (name of the first section)
;                  Failure - 0, sets @error to:
;                  |1 - could not find any files
;                  |2 - could not read any file
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_MI_ListAll(Const $sTransPath, $sExtension='ini')
	; Author: ProgAndy
	If Not IsString($sExtension) Or $sExtension='' Then $sExtension='ini'
	Local $aFiles = _FileListToArray($sTransPath, "*." & $sExtension, 1)
	If @error Then Return SetError(1,0,0)
	Local $aTranslations[$aFiles[0]+1][2], $aSection, $j = 0
	For $i = 1 To $aFiles[0]
		$aSection = IniReadSectionNames($sTransPath & '\' & $aFiles[$i])
		If @error Then ContinueLoop

		$j += 1
		$aTranslations[$j][0] = $aFiles[$i]
		$aTranslations[$j][1] = $aSection[1]
	Next
	$aTranslations[0][0] = $j
	If $j = 0 Then Return SetError(2,0,0)
	Return $aTranslations
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_MI_Load
; Description ...: Loads a language from a file
; Syntax.........: _Translation_MI_Load(Const $sTransPath, $sIni, $sLanguage='')
; Parameters ....: $sTransPath - Path to the language-files
;                  $sIni       - Name of the Ini to load
;                  $sLanguage  - [optional] Name of the language (section) to load
; Return values .: Success - 1
;                  Failure - 0, sets @error to:
;                  |1 - Could not create dictionary object
;                  |2 - Could not load language
; Author ........: ProgAndy
; Modified.......:
; Remarks .......: $sIni can also receive a 1D-Array of the following format: $array[2] = [$sIni, $sLanguage]
;                  as returned from _Translation_MI_Choose
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_MI_Load(Const $sTransPath, $sIni, $sLanguage='')
	; Author: ProgAndy
	If Not IsObj($__goTranslationObject) Then $__goTranslationObject = ObjCreate("Scripting.Dictionary")
	If Not IsObj($__goTranslationObject) Then Return SetError(1,0,0)
	$__goTranslationObject.CompareMode = 1 ; Text
	If UBound($sIni) = 2 And UBound($sIni,0) = 1 Then
		$sLanguage = $sIni[1]
		$sIni = $sIni[0]
	EndIf
	Local $aSection = IniReadSection($sTransPath & '\' & $sIni, $sLanguage)
	If @error Then
		$aSection = IniReadSectionNames($sTransPath & '\' & $sIni)
		If @error Then Return SetError(2,0,0)
		$aSection = IniReadSection($sTransPath & '\' & $sIni, $aSection[1])
		If @error Then Return SetError(2,0,0)
	EndIf
	$__goTranslationObject.RemoveAll( )
	For $i = 1 To $aSection[0][0]
		If not $__goTranslationObject.Exists($aSection[$i][0]) then $__goTranslationObject.Add($aSection[$i][0], $aSection[$i][1])
	Next
	Return SetExtended($aSection[0][0], 1)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name...........: _Translation_MI_Choose
; Description ...: Allows to choose a language from a multiini-setup
; Syntax.........: _Translation_MI_Choose(ByRef Const $aTranslations, $iStart=1)
; Parameters ....: $aTranslations  - 2D Array with translations (like returned from _Translation_MI_ListAll)
;                  $iStart         - [optional] Startindex of array (default: 1)
; Return values .: Success - array with Filename and language name.
;                  Failure - 0, sets @error to:
;                  |1 -
; Author ........: ProgAndy
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Translation_MI_Choose(ByRef Const $aTranslations, $iStart=1)
	; Author: ProgAndy
	If Not (UBound($aTranslations,0) = 2) Or $iStart < 0 Or UBound($aTranslations)<= $iStart Then Return SetError(1,0,0)
	Local $aReturn[2] = [$aTranslations[$iStart][0], $aTranslations[$iStart][1]], $sTranslation
	If UBound($aTranslations)-$iStart = 1 Then Return $aReturn
	Local $iOldOpt = Opt("GUIOnEventMode", 0)
	Local $iOldOptChar = Opt("GUIDataSeparatorChar", Chr(2))
	Local $hGUI = GUICreate("Choose Translation", 200, 100, -1, -1, BitOR($WS_CAPTION, $WS_POPUP, $WS_SYSMENU)), $uMsg, $sTranslation=''
	GUICtrlCreateLabel("Choose translation:", 10, 10, 180, 20)
	Local $iCombo = GUICtrlCreateCombo($aTranslations[$iStart][1], 10, 30, 180, 20)
	For $i = $iStart+1 To UBound($aTranslations)-1
		GUICtrlSetData($iCombo, $aTranslations[$i][1])
	Next
	Opt("GUIDataSeparatorChar", $iOldOptChar)
	Local $btnOK = GUICtrlCreateButton("&OK", 10, 60, 85, 25, 0x0001)
	Local $btnCancel = GUICtrlCreateButton("&Cancel", 105, 60, 85, 25)

	GUISetState()
	While 1
		$uMsg = GUIGetMsg(1)
		If $uMsg[1] = $hGUI Then
			Switch $uMsg[0]
				Case -3, $btnCancel
					ExitLoop
				Case $btnOK
					$sTranslation = GUICtrlRead($iCombo)
					For $i = $iStart+1 To UBound($aTranslations)-1
						If $aTranslations[$i][1] = $sTranslation Then
							Local $aReturn[2] = [$aTranslations[$i][0], $aTranslations[$i][1]]
							ExitLoop
						EndIf
					Next
					ExitLoop
			EndSwitch
		EndIf
	WEnd
	Opt("GUIOnEventMode", $iOldOpt )
	GUIDelete($hGUI)
	Return SetError(IsInt($aReturn), 0, $aReturn)
EndFunc