#region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Icon=script_editor.ico
#AutoIt3Wrapper_Res_Comment=Editor with Lexer completely written in AutoIt, no included LexerLibs used.
#AutoIt3Wrapper_Res_Description=Script Code Editor
#AutoIt3Wrapper_Res_Fileversion=1.9.0.2
#AutoIt3Wrapper_Res_Fileversion_AutoIncrement=y
#AutoIt3Wrapper_Res_LegalCopyright=Holger Pandel, 2014
#AutoIt3Wrapper_Res_requestedExecutionLevel=asInvoker
#AutoIt3Wrapper_Res_Field=AutoIt Version|%AutoItVer%
#AutoIt3Wrapper_Res_Field=Author|Holger Pandel
#AutoIt3Wrapper_Res_Field=Email|holger.pandel@googlemail.com
#AutoIt3Wrapper_Res_Field=Product Name|Simple ScriptEditor
#AutoIt3Wrapper_Res_Field=Product Version|1.9
#AutoIt3Wrapper_Res_Field=AutoIt Version|%AutoItVer%
#AutoIt3Wrapper_Res_Field=Compile Date|%date% %time%
#AutoIt3Wrapper_Run_AU3Check=y
#endregion ;**** Directives created by AutoIt3Wrapper_GUI ****


;**** Directives created by AutoIt3Wrapper_GUI ****
; the skeleton of this editor is made out of components from the Scintilla example by Lokster
; http://www.autoitscript.com/forum/topic/40372-more-complete-example-of-using-scintilla-in-autoit/
;
; but I took out the complete lexer reference and wrote a lexer completely with AutoIt
; see _LexingUDF.au3 for further details


Opt("GUIOnEventMode", 1)
Opt("TrayIconHide", 1)
Opt("MustDeclareVars", 1)

; get additional debug output
Global $Debug = True

; appplication title and version
Global Const $appname = "Simple ScriptEditor"
Global Const $version = "1.9"
Global $title = $appname & " v" & $version
Global $configpath = @AppDataDir & "\ScriptEditor"

; basic gui mode - in GUI_EMBED via cmdline, some menu elements are hidden
Global Enum $GUI_FULL, $GUI_EMBED
Global $gui_mode = $GUI_FULL
Global $cmdline_script = ''

Global $default_language = 'de' ; default program language
Global $lexer = 'none' ; overwrite $__default_lexer from _LexingUDF.au3

;general includes
#include "Libs\_Translate.au3"
#include <GUIConstantsEx.au3>
#include <WindowsConstants.au3>
#include <ButtonConstants.au3>
#include <EditConstants.au3>
#include <MenuConstants.au3>
#include <GUIStatusBar.au3>
#include <WinApi.au3>
#include "Libs\Marquee.au3"
#include "Libs\GetOpt.au3"
#include "_LexingUDF.au3"

; every global variable and GUI element defined, so let's have a look at the command line
; let's have a look at the CmdLine
Global $cmdline_fold = -1 ; -1 sets language default
Global $cmdline_highlight = -1 ; -1 sets language default
If 0 <> $CmdLine[0] Then
	_getopts()
EndIf

; ------------------------------------------------------------------------------------------------------------------------
; --------------- LOAD DEFAULT LEXER -------------------------------------------------------------------------------------
; ------------------------------------------------------------------------------------------------------------------------
;#include "LexerDef\opsi.au3"
;#include "LexerDef\autoit.au3"
;#include "LexerDef\none.au3"

;_GUICtrlLex_WriteLANGFile($lexer)
;Exit

; ------------------------------------------------------------------------------------------------------------------------
; --------------- I18N ---------------------------------------------------------------------------------------------------
; ------------------------------------------------------------------------------------------------------------------------
; i18n init
Global $sPath = @ScriptDir & "\language"
; get translations
Global $aTranslations = _Translation_MI_ListAll($sPath, "sce")
If FileExists($sPath & "\" & $default_language & ".sce") Then
	_Translation_MI_Load($sPath, $default_language & ".sce")
Else
	Local $aLanguage = _Translation_MI_Choose($aTranslations)
	_Translation_MI_Load($sPath, $aLanguage)
EndIf

;Forms
#include "Forms\edGUI.isf"
#include "Forms\frmFind.isf"
#include "Forms\processing.isf"

; ------------ set window titles -------------
WinSetTitle($pleasewait, "", __("Working..."))
WinSetTitle($frmFind, "", __("Find / Replace"))

; resize handler
GUIRegisterMsg($WM_GETMINMAXINFO, "WM_GETMINMAXINFO")
GUIRegisterMsg($WM_DROPFILES, "WM_DROPFILES")

; Subclassing for keyboard monitoring
; siehe: https://www.autoit.de/index.php?page=Thread&postID=342567#post342567
; -> every key event will be redirected to _KeyProc for further processing
Global $hCallback = DllCallbackRegister("_KeyProc", "long", "int;wparam;lparam")
Global $hHook = _WinAPI_SetWindowsHookEx($WH_KEYBOARD_LL, DllCallbackGetPtr($hCallback), _WinAPI_GetModuleHandle(0))

; ------------------------------------------------------------------------------------------------------------------------
; --------------- GLOBAL EDITOR PARAMS AND VARIABLES ---------------------------------------------------------------------
; ------------------------------------------------------------------------------------------------------------------------

; about window handle
Global $frmAbout, $marquee

;Global $user32 = DllOpen("user32.dll")
Global $_FILENAME = "" ;holds the currently opened filename
Global $modWindowCaptionWritten = true ; change mark in window caption - already set = true, has to be set = false, see WM_KEYDOWN

;Options
Global $show_line_numbers = False
Global $show_folds = False
Global $show_whitespaces = False
Global $tab_width = 2 ; tab width in spaces
Global $pixelWidth ; holds pixelwidth of value "99999" for line numbers

;Find
Global $_OPT_FIND_WRAPAROUND = True ; when searching, restart at top of the document?
Global $findWhat = "" ; search term
Global $reverseDirection = False ; downwards/upwards searching
Global $showWarnings = False ; some messages
;$flags = ($wholeWord ? SCFIND_WHOLEWORD : 0) |
;            (matchCase ? SCFIND_MATCHCASE : 0) |
;            (regExp ? SCFIND_REGEXP : 0) |
;            (props.GetInt("find.replace.regexp.posix") ? SCFIND_POSIX : 0);
Global $flags = 0

;program return code for external file change recognition
Global $retcode = 0; if program exits with 555, file changes have just been saved before closing

; ------------------------------------------------------------------------------------------------------------------------
; --------------- GUI EVENTS ---------------------------------------------------------------------------------------------
; ------------------------------------------------------------------------------------------------------------------------

; define the menu elements which are NOT there in embedded mode ;-), otherwise the interpreter throws an error
Global $mFileNew, $mFileOpen, $mFileSaveAs, $mHelpAbout, $mLexerDef, $mChooseLang
; general events
GUISetOnEvent($GUI_EVENT_CLOSE, "CloseFind", $frmFind)
GUISetOnEvent($GUI_EVENT_CLOSE, "Bye", $edGUI)
GUISetOnEvent($GUI_EVENT_RESIZED, 'WM_SIZE', $edGUI)
GUISetOnEvent($GUI_EVENT_MAXIMIZE, 'WM_SIZE', $edGUI)

;File
GUICtrlSetOnEvent($mFile, "MenuCmd")
If $gui_mode = $GUI_FULL Then GUICtrlSetOnEvent($mFileNew, "MenuCmd")
If $gui_mode = $GUI_FULL Then GUICtrlSetOnEvent($mFileOpen, "MenuCmd")
GUICtrlSetOnEvent($mFileSave, "MenuCmd")
GUICtrlSetOnEvent($mFileSaveAs, "MenuCmd")
GUICtrlSetOnEvent($mFileExit, "MenuCmd")
;Edit
GUICtrlSetOnEvent($mEdit, "MenuCmd")
GUICtrlSetOnEvent($mEditUndo, "MenuCmd")
GUICtrlSetOnEvent($mEditRedo, "MenuCmd")
GUICtrlSetOnEvent($mEditCut, "MenuCmd")
GUICtrlSetOnEvent($mEditCopy, "MenuCmd")
GUICtrlSetOnEvent($mEditPaste, "MenuCmd")
GUICtrlSetOnEvent($mEditDelete, "MenuCmd")
GUICtrlSetOnEvent($mEditSelectAll, "MenuCmd")
GUICtrlSetOnEvent($mFoldAll, "MenuCmd")
;Search
GUICtrlSetOnEvent($mSearchFind, "MenuCmd")
GUICtrlSetOnEvent($mSearchFindNext, "MenuCmd")
GUICtrlSetOnEvent($mSearchFindPrevious, "MenuCmd")
GUICtrlSetOnEvent($mSearchGotoLine, "MenuCmd")
;View
GUICtrlSetOnEvent($mViewLineNumbers, "MenuCmd")
GUICtrlSetOnEvent($mViewFolds, "MenuCmd")
GUICtrlSetOnEvent($mViewWhitespaces, "MenuCmd")
GUICtrlSetOnEvent($mStyling, "MenuCmd")
GUICtrlSetOnEvent($mFolding, "MenuCmd")
GUICtrlSetOnEvent($mFoldCompact, "MenuCmd")
GUICtrlSetOnEvent($mFoldComment, "MenuCmd")
GUICtrlSetOnEvent($mCountWords, "MenuCmd")

;Help
If $gui_mode = $GUI_FULL Then GUICtrlSetOnEvent($mHelpAbout, "MenuCmd")

;$dum = GUICtrlCreateDummy()
;HotKeySet("{F1}","_GUICtrlLex_StyleIt")

#region mainloop
; Hey ho, let's go!
$__default_editor_parent = $edGUI
$__styleSelHook = "SetupEditorDefaults"
_GUICtrlLex_ReadLEXFile($lexer, $cmdline_highlight, $cmdline_fold)
_GUICtrlLex_CreateScriptEditor(0, 0, 600, 457)
_Resize_AddWindow($__hWnd_SciLexer, $GUI_DOCKBORDERS)
_GUICtrlLex_SetupStylingAndFolding()
SetupEditorDefaults()
SetKeyPressEvents()
If $cmdline_script <> '' Then
	OpenFile($cmdline_script)
Else
	NewFile()
EndIf

GUISetState(@SW_SHOW, $edGUI)
SendMessage($__hWnd_SciLexer, $SCI_CLEARSELECTIONS, 0, 0)

While 1
	If (WinActive($edGUI)) Then
		; if you want to use a statusbar, uncomment and correct handle variable
		If IsDeclared("statBar") Then _GUICtrlStatusBar_SetText($statBar, __("Line") & "=" & $__currentLine + 1 & " " & __("Column") & "=" & $__currentColumn & " / " & __("Total Lines:") & " " & Sci_GetLineCount($__hWnd_SciLexer))
	EndIf

	Sleep(100)
WEnd
#endregion mainloop

; ------------------------------------------------------------------------------------------------------------------------
; ----------------- EDITOR AND MENU HANDLING -----------------------------------------------------------------------------
; ------------------------------------------------------------------------------------------------------------------------

; set key event handler

Func SetKeyPressEvents()
	HotKeySet("{ESC}", "keyEventHandler") ; Close windows / exit program
	HotKeySet("{F3}", "keyEventHandler") ; F3 - Find next
	HotKeySet("{F1}", "keyEventHandler") ; Re-Lex if $debug = true
	HotKeySet("^s", "keyEventHandler") ; Ctrl-S - Save
	HotKeySet("^+s", "keyEventHandler") ; Ctrl-Shift-S - Save as
	HotKeySet("^o", "keyEventHandler") ; Ctrl-O - Open
	HotKeySet("^n", "keyEventHandler") ; Ctrl-N - New
	HotKeySet("^g", "keyEventHandler") ; Ctrl-G - Goto line
	HotKeySet("^f", "keyEventHandler") ; Ctrl-F - Find
EndFunc   ;==>SetKeyPressEvents

; delete key events
Func UnsetKeyPressEvents()
	HotKeySet("{ESC}")
	HotKeySet("{F3}")
	HotKeySet("{F1}")
	HotKeySet("^s")
	HotKeySet("^+s")
	HotKeySet("^o")
	HotKeySet("^n")
	HotKeySet("^g")
	HotKeySet("^f")
EndFunc   ;==>UnsetKeyPressEvents

Func keyEventHandler()
	If Not IsDeclared("frmLexDef") Then Local $frmLexDef = -1 ; form isn't defined all the time, so define if necessary
	If WinActive($edGUI) Or WinActive($frmFind) Or WinActive($frmLexDef) Then
		Switch @HotKeyPressed
			Case "{ESC}"
				Select
					Case WinActive($edGUI)
						Bye()
					Case WinActive($frmFind)
						CloseFind()
					Case WinActive($frmLexDef)
						_GUICtrlLex_CloseLexerDefForm()
				EndSelect
			Case "{F3}"
				FindNext($findWhat, True, $showWarnings, $flags, False)
			Case "{F1}"
				If $Debug Then
					_GUICtrlLex_StyleIt(0, Sci_GetLength($__hWnd_SciLexer))
					_GUICtrlLex_FoldIt(0, Sci_GetLength($__hWnd_SciLexer))
				EndIf
			Case "^+s"
					SaveFile()
			Case "^s"
					SaveFile($_FILENAME)
			Case "^o"
				OpenFile()
			Case "^n"
				NewFile()
			Case "^g"
				GoToLine()
			Case "^f"
				$findWhat = Sci_GetSelectionText($__hWnd_SciLexer)
				GUICtrlSetData($Combo1, $findWhat, $findWhat)
				FindNext($findWhat, False, $showWarnings, $flags, True)
		EndSwitch
	EndIf
EndFunc   ;==>keyEventHandler

; handle command line options

Func _getopts()
	Local $sMsg = '------------------------------------------------------------------------------------------------------------------' & @CRLF ; start line
	$sMsg &= $title & ' Command Line' & @CRLF & 'Parsing: ' & _ArrayToString($CmdLine, ' ', 1) & @CRLF & @CRLF ; Message.
	Local $sOpt, $sSubOpt, $sOper
	Local $h_specified = False
	Local $failure = False
	; Options array, entries have the format [short, long, default value]
	Local $aOpts[8][3] = [ _
			['-p', '--path', $GETOPT_REQUIRED_ARGUMENT], _
			['-l', '--lang', $GETOPT_REQUIRED_ARGUMENT], _
			['-x', '--lexer', $GETOPT_REQUIRED_ARGUMENT], _
			['-f', '--fold', True], _
			['-s', '--style', True], _
			['-e', '--embed', True], _
			['-h', '--help', True], _
			['-d', '--debug', True]]


	_GetOpt_Set($aOpts) ; Set options.

	If 0 < $GetOpt_Opts[0] Then ; If there are any options...

		; first loop: analyse parameters and create message texts, exit program if needed
		While 1 ; ...loop through them one by one.
			; Get the next option passing a string with valid options.
			$sOpt = _GetOpt('p:l:x:efshd') ; p: means -p option requires an argument.
			If Not $sOpt Then ExitLoop ; No options or end of loop.
			; Check @extended above if you want better error handling.
			; The current option is stored in $GetOpt_Opt, it's index (in $GetOpt_Opts)
			; in $GetOpt_Ind and it's value in $GetOpt_Arg.
			Switch $sOpt ; What is the current option?
				Case '?' ; Unknown options come here. @extended is set to $E_GETOPT_UNKNOWN_OPTION
					$sMsg &= 'Unknown option: ' & $GetOpt_Ind & ': ' & $GetOpt_Opt
					$sMsg &= ' with value "' & $GetOpt_Arg & '" (' & VarGetType($GetOpt_Arg) & ').' & @CRLF
					$sMsg &= 'UNKNOWN OPTION passed.' & @CRLF
					$failure = True
				Case ':' ; Options with missing required arguments come here. @extended is set to $E_GETOPT_MISSING_ARGUMENT
					$sMsg &= 'MISSING REQUIRED ARGUMENT for option: ' & $GetOpt_Ind & ': ' & $GetOpt_Opt & @CRLF
					$failure = True
				Case 'e', 'd', 'f', 's'
					$sMsg &= 'Option ' & $GetOpt_Ind & ': ' & $GetOpt_Opt
					$sMsg &= ' with value "' & $GetOpt_Arg & '" (' & VarGetType($GetOpt_Arg) & ')'
					$sMsg &= '.' & @CRLF
					Switch $GetOpt_Opt ; What is the current option?
						Case "d"
							$Debug = True
						Case "e"
							$gui_mode = $GUI_EMBED
						Case "f"
							$cmdline_fold = 1
						Case "s"
							$cmdline_highlight = 1
					EndSwitch
				Case 'p'
					$sMsg &= 'Option ' & $GetOpt_Ind & ': ' & $GetOpt_Opt
					$sMsg &= ' with required value "' & $GetOpt_Arg & '" (' & VarGetType($GetOpt_Arg) & ')'
					$sMsg &= '.' & @CRLF
					$cmdline_script = $GetOpt_Arg
				Case 'l'
					$sMsg &= 'Option ' & $GetOpt_Ind & ': ' & $GetOpt_Opt
					$sMsg &= ' with required value "' & $GetOpt_Arg & '" (' & VarGetType($GetOpt_Arg) & ')'
					$sMsg &= '.' & @CRLF
					$default_language = $GetOpt_Arg
				Case 'x'
					$sMsg &= 'Option ' & $GetOpt_Ind & ': ' & $GetOpt_Opt
					$sMsg &= ' with required value "' & $GetOpt_Arg & '" (' & VarGetType($GetOpt_Arg) & ')'
					$sMsg &= '.' & @CRLF
					$lexer = $GetOpt_Arg
				Case 'h'
					$sMsg &= 'Available command line options:' & @CRLF & @CRLF & _
							'-p' & @TAB & '--path ' & @TAB & 'Path to script file' & @CRLF & _
							'-e' & @TAB & '--embed' & @TAB & 'Embedded mode' & @CRLF & _
							'-f' & @TAB & '--fold' & @TAB & 'Turn on folding mode' & @CRLF & _
							'-s' & @TAB & '--style' & @TAB & 'Turn on syntax highlighting' & @CRLF & _
							'-l' & @TAB & '--lang' & @TAB & 'Editor language (filename without extension, see subfolder "language"' & @CRLF & _
							'-x' & @TAB & '--lexer' & @TAB & 'Default Lexer (filename without extension, see subfolder "LexerDef"' & @CRLF & _
							'-d' & @TAB & '--debug' & @TAB & 'Write additional debug output (can create very much text!!)' & @CRLF & _
							'-h' & @TAB & '--help ' & @TAB & 'Show this help' & @CRLF & @CRLF & _
							'Example 1:' & @TAB & 'scripteditor.exe --path=d:\scripts\test.ins -x=autoit --embed --style -l=de' & @CRLF & _
							'Example 2:' & @TAB & 'scripteditor.exe -e -s --lexer=autoit -l=de d:\scripts\test.ins' & @CRLF & _
							@TAB & 'This starts Script Editor with german language in embedded mode, opens d:\scripts\test.ins' & @CRLF & _
							@TAB & 'for editing, loads the AutoIt lexer preference file and turns syntax highlighting on.'
					$h_specified = True
			EndSwitch
		WEnd

		; check for operands, there shouldn't be ANY of them
		;$sMsg &= @CRLF
		If 0 < $GetOpt_Opers[0] Then ; If there are any operands...
			If $GetOpt_Opers[0] > 1 Then
				$sMsg &= @CRLF & 'INVALID OR TOO MANY OPERANDS PASSED. This is not allowed. Check for missing ''='' in --path or --lang option!' & @CRLF
				While 1 ; ...loop through them one by one.
					$sOper = _GetOpt_Oper() ; Get the next operand.
					If Not $sOper Then ExitLoop ; no operands or end of loop.
					; Check @extended above if you want better error handling.
					$sMsg &= 'Operand ' & $GetOpt_OperInd & ': ' & $sOper & @CRLF
				WEnd
				$failure = True
			Else
				$cmdline_script = _GetOpt_Oper()
			EndIf
		EndIf

		If $failure Then $sMsg &= @CRLF & 'Please check the command line parameters!' & @CRLF

		If $h_specified Or $failure Then ; show help screen or wrong options ?
			MsgBox(262208, __("Warning"), $sMsg, 0)
			Exit
		EndIf
	EndIf
EndFunc   ;==>_getopts

; some editor defaults at startup

Func SetupEditorDefaults()
	; setup defaults
	_showLineNumbers($show_line_numbers)
	_GUICtrlStatusBar_SetText($statBar, "")
	SendMessage($Sci, $SCI_SETTABWIDTH, $tab_width, 0)
	_showWhitespaces($show_whitespaces)
	_showFolds($show_folds)
	_showFolding()
	_showStyling()
EndFunc   ;==>SetupEditorDefaults

; handle every menu command

Func MenuCmd()
	Switch @GUI_CtrlId
		Case $mFileNew
			NewFile()
		Case $mFileOpen
			OpenFile()
		Case $mFileSave
			SaveFile($_FILENAME)
		Case $mFileSaveAs
			SaveFile()
		Case $mSearchFind
			GUISetState(@SW_SHOW, $frmFind)
		Case $mSearchFindNext
			FindNext($findWhat, False, $showWarnings, $flags, False)
		Case $mSearchFindPrevious
			FindNext($findWhat, True, $showWarnings, $flags, False)
		Case $mEditUndo
			Sci_Undo($__hWnd_SciLexer)
		Case $mEditRedo
			Sci_Redo($__hWnd_SciLexer)
		Case $mEditCut
			Sci_Cut($__hWnd_SciLexer)
		Case $mEditCopy
			Sci_Copy($__hWnd_SciLexer)
		Case $mEditPaste
			Sci_Paste($__hWnd_SciLexer)
		Case $mEditDelete
			Sci_Clear($__hWnd_SciLexer)
		Case $mEditSelectAll
			Sci_SelectAll($__hWnd_SciLexer)
		Case $mFoldAll
			Sci_FoldAll($__hWnd_SciLexer)
		Case $mSearchGotoLine
			GoToLine()
		Case $mViewLineNumbers
			$show_line_numbers = Not $show_line_numbers
			_showLineNumbers($show_line_numbers)
		Case $mViewWhitespaces
			$show_whitespaces = Not $show_whitespaces
			_showWhitespaces($show_whitespaces)
		Case $mViewFolds
			$show_folds = Not $show_folds
			_showFolds($show_folds)
		Case $mFolding
			GUISetState(@SW_SHOW, $pleasewait)
			$__fold = Not $__fold
			_showFolding()
			GUISetState(@SW_HIDE, $pleasewait)
		Case $mFoldCompact
			GUISetState(@SW_SHOW, $pleasewait)
			$__foldCompact = Not $__foldCompact
			_showFolding()
			GUISetState(@SW_HIDE, $pleasewait)
		Case $mFoldComment
			GUISetState(@SW_SHOW, $pleasewait)
			$__foldComment = Not $__foldComment
			_GUICtrlLex_SetupFoldingArray()
			_showFolding()
			GUISetState(@SW_HIDE, $pleasewait)
		Case $mStyling
			GUISetState(@SW_SHOW, $pleasewait)
			$__highlight = Not $__highlight
			_showStyling()
			GUISetState(@SW_HIDE, $pleasewait)
		Case $mHelpAbout
			_showAbout()
		Case $mCountWords
			Local $text = Sci_GetTextUTF8WithoutNull($__hWnd_SciLexer)
			Local $array = StringRegExp($text, "[\s,;:\.]*([[:alnum:]]+)[\s,;:\.]*", 3)
			MsgBox(0, __("Word count"), __("Total number of words in document:") & " " & UBound($array), 0, $edGUI)
		Case $mFileExit
			Bye()
	EndSwitch
EndFunc   ;==>MenuCmd

; close find dialog

Func CloseFind()
	GUISetState(@SW_HIDE, $frmFind)
EndFunc   ;==>CloseFind

; close about popup

Func CloseAbout()
	GUISetState(@SW_HIDE, $frmAbout)
	_GUICtrlMarquee_Delete($marquee)
	GUIDelete($frmAbout)
EndFunc   ;==>CloseAbout

; close prog

Func Bye()
	If Sci_GetModify($__hWnd_SciLexer) <> 0 Then
		Local $ans = MsgBox(3 + 32, __("Script Editor"), __("Save changes to file") & " " & $_FILENAME & " ?", 0, $edGUI)
		Switch $ans
			Case 2 ; cancel
				Return
			Case 6 ;yes
				SaveFile($_FILENAME)
				$retcode = 555
			Case 7 ;no
		EndSwitch
	EndIf
    _WinAPI_UnhookWindowsHookEx($hHook)
    DllCallbackFree($hCallback)
    GUIDelete()
	Exit $retcode
EndFunc   ;==>Bye

; open a file for editing

Func OpenFile($from = '')
	If $from = '' Then
		If Sci_GetModify($__hWnd_SciLexer) <> 0 Then
			Local $ans = MsgBox(3 + 32, __("Confirm"), __("Save changes to file") & " " & $_FILENAME & " ?", 0, $edGUI)
			Switch $ans
				Case 2 ; cancel
					Return
				Case 6 ;yes
					SaveFile($_FILENAME)
			EndSwitch
		EndIf

		$from = FileOpenDialog(__("Open file..."), "", $__filetypes, 3, "", $edGUI)
		If $from = '' Then
			Return
		EndIf
	EndIf

	$_FILENAME = $from

	Local $file = FileOpen($from, 0 + 256) ; read mode (0) + full UTF-8 without BOM (256)
	; Check if file opened for reading OK
	If $file = -1 Then
		MsgBox(262160, __("Error"), __("Error: Unable to open file."), 0)
		Return
	EndIf

	GUISetState(@SW_SHOW, $pleasewait)
	Sci_DelLines($__hWnd_SciLexer)
	Sci_EmptyUndoBuffer($__hWnd_SciLexer)
	Sci_SetSavePoint($__hWnd_SciLexer)

	Sci_Cancel($__hWnd_SciLexer)
	Sci_SetUndoCollection($__hWnd_SciLexer, 0)

	$__lexRunning = True
	While 1
		Local $buffer = FileRead($file, 4095)
		If @error = -1 Then ExitLoop
		Sci_AppendText($__hWnd_SciLexer, _convertUTF8ToMultiByte($buffer))
	WEnd
	FileClose($file)
	Sci_SetUndoCollection($__hWnd_SciLexer)
	Sci_EmptyUndoBuffer($__hWnd_SciLexer)
	Sci_SetSavePoint($__hWnd_SciLexer)
	Sci_SetCurrentPos($__hWnd_SciLexer, 0)
	_setTitel()
	$modWindowCaptionWritten = false
	; send message to recolourise the whole text once
	_GUICtrlLex_StyleIt()
	_GUICtrlLex_FoldIt()
	GUISetState(@SW_HIDE, $pleasewait)
	;	SendMessage($__hWnd_SciLexer, $SCI_COLOURISE, 0, -1)
	$__lexRunning = False
	SendMessage($__hWnd_SciLexer, $SCI_CLEARSELECTIONS, 0, 0)
EndFunc   ;==>OpenFile

; save file to existing path name

Func SaveFile($fn = "")
	If $fn = '' Then
		If $_FILENAME = "" Then
			$fn = _FileSaveDialog(__("Save file..."), "", $__filetypes, 3, "newscript", "", $edGUI)
		Else
			$fn = _FileSaveDialog(__("Save file..."), "", $__filetypes, 3, $_FILENAME, "", $edGUI)
		EndIf
	EndIf

	If $fn <> '' Then
		$_FILENAME = $fn
		Local $text = Sci_GetTextUTF8WithoutNull($Sci)
		Local $handle = FileOpen($_FILENAME, 2 + 256) ; write mode (2) + full UTF-8 without BOM (256)
		FileWrite($handle, $text)
		FileClose($handle)
		$text = 0
		Sci_SetUndoCollection($__hWnd_SciLexer)
		Sci_EmptyUndoBuffer($__hWnd_SciLexer)
		Sci_SetSavePoint($__hWnd_SciLexer)
		_setTitel()
		$modWindowCaptionWritten = false
		Return True
	Else
		Return False
	EndIf

EndFunc   ;==>SaveFile

; setup editor for a new file

Func NewFile()
	If Sci_GetModify($__hWnd_SciLexer) <> 0 Then
		Local $ans = MsgBox(3 + 32, __("Confirm"), __("Save changes to file") & " " & $_FILENAME, 0, $edGUI)
		Switch $ans
			Case 2 ; cancel
				Return
			Case 6 ;yes
				SaveFile($_FILENAME)
			Case 7 ;no
		EndSwitch
	EndIf

	$__lexRunning = True
	Sci_DelLines($__hWnd_SciLexer)
	Sci_EmptyUndoBuffer($__hWnd_SciLexer)
	Sci_SetSavePoint($__hWnd_SciLexer)
	Sci_SetCurrentPos($__hWnd_SciLexer, 0)
	Sci_SetUndoCollection($__hWnd_SciLexer)
	$_FILENAME = ""
	_setTitel()
	; send message to initialize vars
	_GUICtrlLex_StyleIt()
	_GUICtrlLex_FoldIt()
	$__lexRunning = False
EndFunc   ;==>NewFile

; jump to special line

; drop file into editor

Func DropFile()
	ConsoleWrite("blablabl")
	;MsgBox(0, '@GUI_DropID: ' & @GUI_DropID, '@GUI_DragFile: ' & @GUI_DragFile)
	;Local $file = GUICtrlRead(@GUI_DragFile)
	;OpenFile($file)
EndFunc   ;==>DropFile

Func GoToLine()
	Local $line = InputBox(__("Go to line"), __("Enter line number"), "", "", -1, 40)
	If StringIsInt($line) Then
		Sci_GotoLine($__hWnd_SciLexer, $line - 1)
	EndIf
EndFunc   ;==>GoToLine

; find / replace dialog: basic search routine

Func FindNext($findWhat, $reverseDirection = False, $showWarnings = True, $flags = 0, $showgui = False)
	Local $findTarget, $wrapFind, $replacing, $havefound
	If ($findWhat == "") Or ($showgui) Then
		If $reverseDirection Then
			GUICtrlSetState($rFindDirectionUp, $GUI_CHECKED)
			GUICtrlSetState($rFindDirectionDown, $GUI_UNCHECKED)
		Else
			GUICtrlSetState($rFindDirectionUp, $GUI_UNCHECKED)
			GUICtrlSetState($rFindDirectionDown, $GUI_CHECKED)

		EndIf
		GUISetState(@SW_SHOW, $frmFind)
		Return -1
	EndIf

	$findTarget = $findWhat
	Local $lenFind = StringLen($findTarget)
	If ($lenFind == 0) Then
		Return -1
	EndIf

	Local $startPosition = Sci_GetTargetEnd($__hWnd_SciLexer)
	Local $endPosition = Sci_GetLength($__hWnd_SciLexer)
	If ($reverseDirection) Then
		$startPosition = Sci_GetTargetStart($__hWnd_SciLexer)
		$endPosition = 0
	EndIf

	Sci_SetSearchFlags($__hWnd_SciLexer, $flags)

	Local $posFind = Sci_Search($__hWnd_SciLexer, $findTarget, $startPosition, $endPosition)

	If ($posFind == -1) And ($wrapFind) Then
		If ($reverseDirection) Then
			$startPosition = Sci_GetLength($__hWnd_SciLexer)
			$endPosition = 0;
		Else
			$startPosition = 0;
			$endPosition = Sci_GetLength($__hWnd_SciLexer)
		EndIf
		$posFind = Sci_Search($__hWnd_SciLexer, $findTarget, $startPosition, $endPosition)
		If ($showWarnings) Then
			MsgBox(262144 + 8192, __("Warning"), __("Reached the end of text. Wrapping around..."));
		EndIf
	EndIf
	If ($posFind == -1) Then
		$havefound = False;
		If ($showWarnings) Then
			MsgBox(262144 + 8192, __("Warning"), __("Can't find the string") & " '" & $findWhat & "'!");
		EndIf
	Else
		$havefound = True;
		Local $start = Sci_GetTargetStart($__hWnd_SciLexer)
		Local $end = Sci_GetTargetEnd($__hWnd_SciLexer)
		Sci_EnsureRangeVisible($__hWnd_SciLexer, $start, $end, True);
		Sci_SetSelection($__hWnd_SciLexer, $start, $end)
	EndIf
	Return $posFind;
EndFunc   ;==>FindNext

; find / replace dialog: find next word button handler

Func btnFindNextClick()
	Global $Sci, $flags, $findWhat, $findTarget, $wrapFind, $reverseDirection
	$flags = 0

	If GUICtrlRead($cbFindMatchCase) == $GUI_CHECKED Then
		$flags = BitOR($flags, $SCFIND_MATCHCASE)
	EndIf

	If GUICtrlRead($cbFindWholeWords) == $GUI_CHECKED Then
		$flags = BitOR($flags, $SCFIND_WHOLEWORD)
	EndIf

	If GUICtrlRead($cbFindRe) == $GUI_CHECKED Then
		$flags = BitOR($flags, $SCFIND_REGEXP, $SCFIND_POSIX)
	EndIf

	If GUICtrlRead($cbFindWrapAround) == $GUI_CHECKED Then
		$wrapFind = True
	Else
		$wrapFind = False
	EndIf


	If GUICtrlRead($rFindDirectionUp) == $GUI_CHECKED Then
		$reverseDirection = True
	Else
		$reverseDirection = False
	EndIf

	If GUICtrlRead($cbFindShowWarnings) == $GUI_CHECKED Then
		$showWarnings = True
	Else
		$showWarnings = False
	EndIf

	$findWhat = GUICtrlRead($Combo1)
	GUICtrlSetData($Combo1, $findWhat, $findWhat)
	FindNext(GUICtrlRead($Combo1), $reverseDirection, $showWarnings, $flags)
EndFunc   ;==>btnFindNextClick

; find / replace dialog: replace ALL at once

Func btnFindReplaceAllClick()
	Global $Sci, $flags, $findWhat, $findTarget, $wrapFind, $reverseDirection
	$flags = 0

	If GUICtrlRead($cbFindMatchCase) == $GUI_CHECKED Then
		$flags = BitOR($flags, $SCFIND_MATCHCASE)
	EndIf

	If GUICtrlRead($cbFindWholeWords) == $GUI_CHECKED Then
		$flags = BitOR($flags, $SCFIND_WHOLEWORD)
	EndIf

	If GUICtrlRead($cbFindRe) == $GUI_CHECKED Then
		$flags = BitOR($flags, $SCFIND_REGEXP, $SCFIND_POSIX)
	EndIf

	If GUICtrlRead($cbFindWrapAround) == $GUI_CHECKED Then
		$wrapFind = True
	Else
		$wrapFind = False
	EndIf


	If GUICtrlRead($rFindDirectionUp) == $GUI_CHECKED Then
		$reverseDirection = True
	Else
		$reverseDirection = False
	EndIf

	If GUICtrlRead($cbFindShowWarnings) == $GUI_CHECKED Then
		$showWarnings = True
	Else
		$showWarnings = False
	EndIf

	$findWhat = GUICtrlRead($Combo1)
	While FindNext($findWhat, $reverseDirection, $showWarnings, $flags) > -1
		Sci_ReplaceSel($__hWnd_SciLexer, GUICtrlRead($Combo2))
	WEnd
EndFunc   ;==>btnFindReplaceAllClick

; find / replace dialog: single replace action
Func btnFindReplaceClick()
	If Sci_GetSelectionLen($__hWnd_SciLexer) > 0 Then
		Sci_ReplaceSel($__hWnd_SciLexer, GUICtrlRead($Combo2))
		btnFindNextClick()
	EndIf
EndFunc   ;==>btnFindReplaceClick

; set document title

Func _setTitel($mod = false)
	If $_FILENAME <> "" Then
		If $mod Then
			WinSetTitle($edGUI, "", $_FILENAME & "* - " & $title)
		Else
			WinSetTitle($edGUI, "", $_FILENAME & " - " & $title)
		EndIf
	Else
		WinSetTitle($edGUI, "", __("New") & "* - " & $title)
	EndIf
EndFunc   ;==>_setTitel

; show hide margin with line numbers

Func _showLineNumbers($yes)
	$show_line_numbers = $yes
	If $yes Then
		GUICtrlSetState($mViewLineNumbers, $GUI_UNCHECKED)
		SendMessage($__hWnd_SciLexer, $SCI_SETMARGINWIDTHN, 1, 10)
		SendMessage($__hWnd_SciLexer, $SCI_SETMARGINWIDTHN, 0, 0)
	Else
		GUICtrlSetState($mViewLineNumbers, $GUI_CHECKED)
		SendMessage($__hWnd_SciLexer, $SCI_SETMARGINWIDTHN, 0, $pixelWidth);
		SendMessage($__hWnd_SciLexer, $SCI_SETMARGINWIDTHN, 1, 0)
	EndIf
EndFunc   ;==>_showLineNumbers

; show / hide special characters
Func _showWhitespaces($yes)
	$show_whitespaces = $yes
	If $yes Then
		GUICtrlSetState($mViewWhitespaces, $GUI_CHECKED)
		SendMessage($__hWnd_SciLexer, $SCI_SETVIEWWS, 1, 0)
	Else
		GUICtrlSetState($mViewWhitespaces, $GUI_UNCHECKED)
		SendMessage($__hWnd_SciLexer, $SCI_SETVIEWWS, 0, 0)
	EndIf
EndFunc   ;==>_showWhitespaces

; show / hide document margin with folding symbols

Func _showFolds($yes)
	$show_folds = $yes
	If $yes Then
		GUICtrlSetState($mViewFolds, $GUI_UNCHECKED)
		SendMessage($__hWnd_SciLexer, $SCI_SETMARGINWIDTHN, 2, 0)

	Else
		GUICtrlSetState($mViewFolds, $GUI_CHECKED)
		SendMessage($__hWnd_SciLexer, $SCI_SETMARGINWIDTHN, 2, 15);
	EndIf
EndFunc   ;==>_showFolds

; set state of menu items and recalculate folding for whole document

Func _showFolding()
	If Not $__fold Then
		GUICtrlSetState($mFolding, $GUI_UNCHECKED)
		GUICtrlSetState($mFoldCompact, $GUI_DISABLE)
		GUICtrlSetState($mFoldComment, $GUI_DISABLE)
		GUICtrlSetState($mFoldAll, $GUI_DISABLE)
	Else
		GUICtrlSetState($mFolding, $GUI_CHECKED)
		GUICtrlSetState($mFoldCompact, $GUI_ENABLE)
		GUICtrlSetState($mFoldComment, $GUI_ENABLE)
		GUICtrlSetState($mFoldAll, $GUI_ENABLE)
		If Not $__foldCompact Then
			GUICtrlSetState($mFoldCompact, $GUI_UNCHECKED)
		Else
			GUICtrlSetState($mFoldCompact, $GUI_CHECKED)
		EndIf
		If Not $__foldComment Then
			GUICtrlSetState($mFoldComment, $GUI_UNCHECKED)
		Else
			GUICtrlSetState($mFoldComment, $GUI_CHECKED)
		EndIf
	EndIf
	; send message to recalculate folding for the whole text once
	$__lexRunning = True ; fake document load to speed up recalculation
	_GUICtrlLex_FoldIt()
	$__lexRunning = False
EndFunc   ;==>_showFolding

; turn syntax highlighting on or off

Func _showStyling()
	If Not $__highlight Then
		GUICtrlSetState($mStyling, $GUI_UNCHECKED)
	Else
		GUICtrlSetState($mStyling, $GUI_CHECKED)
	EndIf
	; send message to recolourise the whole text once
	$__lexRunning = True ; fake document load to speed up recalculation
	_GUICtrlLex_StyleIt()
	$__lexRunning = True
EndFunc   ;==>_showStyling

; about popup

Func _showAbout()
	Local $newline = "<br>"
	$frmAbout = GUICreate("About", 450, 400, -1, -1, $WS_POPUP + $WS_CAPTION + $WS_SYSMENU + $DS_MODALFRAME + $DS_SETFOREGROUND, -1, $edGUI)
	GUISetBkColor(0xFFFFFF, $frmAbout)
	GUICtrlCreateLabel("Script Editor (MIT licensed)", 10, 20, 380, 26, 1, -1)
	GUICtrlSetFont(-1, 12, 700, 0, "MS Sans Serif")
	GUICtrlSetBkColor(-1, "-2")
	GUICtrlCreateLabel("(Version)", 10, 60, 380, 15, 1, -1)
	GUICtrlSetBkColor(-1, "-2")
	GUICtrlSetData(-1, "v" & $version)
	GUICtrlCreateLabel("Holger Pandel, 2013", 10, 80, 380, 15, 1, -1)
	GUICtrlSetBkColor(-1, "-2")
	GUISetOnEvent($GUI_EVENT_CLOSE, "CloseAbout", $frmAbout)

	Local $about = "<center>" & _
			"Thanks for your great software!" & $newline & _
			$newline & _
			"AutoIt v3" & $newline & _
			"http://www.autoitscript.com/site/autoit/" & $newline & _
			$newline & _
			"ISN AutoIt Studio" & $newline & _
			"http://isnetwork.isi-webportal.at/index.php/meine-programme/isn-autoit-studio" & $newline & _
			$newline & _
			"SciLexer UDF by Kip" & $newline & _
			"http://www.autoitscript.com/forum/topic/51547-scilexer-udf/" & $newline & _
			$newline & _
			"Additionally, necessary Scintilla functions added by me and gathered from Prog@ndy" & $newline & _
			"-> see _SciLexer_Addon.au3" & $newline & _
			$newline & _
			"Main menu framework based on Ledit by Lokster" & $newline & _
			"http://www.autoitscript.com/forum/topic/40372-more-complete-example-of-using-scintilla-in-autoit/" & $newline & _
			$newline & _
			"Translate UDF by Prog@ndy" & $newline & _
			"http://www.autoit.de/index.php?page=Thread&threadID=19884" & $newline & _
			$newline & _
			"ArrayAlternatives UDF by Bugfix / AspirinJunkie" & $newline & _
			"http://autoit.de/index.php?page=Thread&threadID=5392" & $newline & _
			$newline & _
			"Marquee UDF by Melba23" & $newline & _
			"http://www.autoitscript.com/forum/index.php?showtopic=143711" & $newline & _
			$newline & _
			"_Resize UDF by TommyDDR" & $newline & _
			"http://autoitscript.fr/forum/viewtopic.php?f=21&t=9252" & $newline & _
			$newline & _
			"ListView Coloring UDF by BugFix" & $newline & _
			"http://www.autoitscript.com/forum/topic/89654-listview-for-every-subitem-set-color-backcol-and-font/" & $newline & _
			$newline & _
			"Font UDF by Simucal <simucal@gmail.com>" & $newline & _
			"http://www.autoitscript.com/forum/topic/25496-font-info-udf/" & $newline & _
			$newline & _
			"IniArray UDF by Raupi" & $newline & _
			"http://ipv4.autoit.de/index.php?page=Thread&postID=152417" & $newline & _
			$newline & _
			"... and other small snippets offered by nice people!" & _
			"</center>"

	$marquee = _GUICtrlMarquee_Init()

	_GUICtrlMarquee_SetScroll($marquee, Default, Default, "up", 2, 100)
	_GUICtrlMarquee_SetDisplay($marquee, 0, Default, "white", 9, "ms sans serif")
	_GUICtrlMarquee_Create($marquee, $about, 10, 80, 430, 250, "About Script Editor")

	GUISetState(@SW_SHOW, $frmAbout)
EndFunc   ;==>_showAbout

; ------------------------------------------------------------------------------------------------------------------------
; ----------------- LITTLE HELPER ----------------------------------------------------------------------------------------
; ------------------------------------------------------------------------------------------------------------------------

Func WM_GETMINMAXINFO($hWnd, $Msg, $wParam, $lParam)
	Local $tagMaxinfo = DllStructCreate("int;int;int;int;int;int;int;int;int;int", $lParam)
	DllStructSetData($tagMaxinfo, 7, 500) ; minimal height X
	DllStructSetData($tagMaxinfo, 8, 400) ; minimal width Y
	;DllStructSetData($tagMaxinfo, 9, $max_width )
	;DllStructSetData($tagMaxinfo, 10, $max_height )
	_GUICtrlStatusBar_Resize($statBar)
	Return 0
EndFunc   ;==>WM_GETMINMAXINFO

Func WM_SIZE()
	_GUICtrlStatusBar_Resize($statBar)
EndFunc   ;==>WM_SIZE

; Author UEZ@Autoit.de / modified: Holger Pandel
Func WM_DROPFILES($hWnd, $Msg, $wParam, $lParam)
	Local $tBuffer = DllStructCreate("char[256]")
	Local $iString

	;Get dropped items count
	Local $aRet = DllCall("shell32.dll", "int", "DragQueryFile", "int", $wParam, "int", -1, "ptr", 0, "int", 0)

	;Getting name only from 1st dropped item
	DllCall("shell32.dll", "int", "DragQueryFile", "int", $wParam, "int", 0, "ptr", DllStructGetPtr($tBuffer), "int", DllStructGetSize($tBuffer))
	$iString = DllStructGetData($tBuffer, 1)
	DllCall("shell32.dll", "none", "DragFinish", "int", $wParam)

	NewFile()
	OpenFile($iString)

	Return $GUI_RUNDEFMSG
EndFunc   ;==>WM_DROPFILES

; Author UEZ@Autoit.de
; alternative key event processing
Func _KeyProc($nCode, $wParam, $lParam) ;subclassing
	; editor window inactive? then get out of here...
	If Not WinActive($edGUI) Then Return _WinAPI_CallNextHookEx($hHook, $nCode, $wParam, $lParam)
    If $nCode < 0 Then Return _WinAPI_CallNextHookEx($hHook, $nCode, $wParam, $lParam)
    Local $tKEYHOOKS = DllStructCreate($tagKBDLLHOOKSTRUCT, $lParam)
    Switch $wparam
        Case $WM_KEYUP
			; if a normal key was pressed then assume that the document has been changed
			; THEN change the window titel and mark it with changes-not-written (*) - but only then ;-)
			If 	$modWindowCaptionWritten = false Then
				If Sci_GetModify($__hWnd_SciLexer) <> 0 Then
					_setTitel(true)
					$modWindowCaptionWritten = True
				EndIf
			EndIf
			;If DllStructGetData($tKEYHOOKS, "vkCode") = 0x0D Then ConsoleWrite("Enter wurde losgelassen." & @CRLF)
        Case $WM_KEYDOWN
			;If DllStructGetData($tKEYHOOKS, "vkCode") = 0x0D Then ConsoleWrite("Enter wurde gedrckt." & @CRLF)
    EndSwitch
    Return _WinAPI_CallNextHookEx($hHook, $nCode, $wParam, $lParam)
EndFunc

